set more off
global direct = "...\CodeLCPC"

use "$direct\FinalHhData.dta", clear

replace year = year+1900 if year >= 80
replace year = year+2000 if year < 04

drop if educ < 9 & year < 1996
drop if educ < 9 & year >= 1996

drop if age < 25 | age > 78
replace famsize = 10 if famsize >= 10

/// deflate: (by region and quarter) CPI CEX region annual deflator
sort year quarter region
merge year quarter region using "$direct\CPIRegionCEX.dta"
drop if _merge == 2
drop _merge
replace cpi = cpi/100
replace cons = cons/cpi
replace income = income/cpi

/// use regional unemployment rate
sort year quarter region
merge year quarter region using "$direct\UnemplRegionCEX.dta"
drop if _merge == 2
drop _merge

generate ret = 0
replace ret = 1 if occup == 10

describe, varlist
foreach var of varlist * {
drop if `var' == .
}

generate cohort = year-age
sum cohort
replace cohort = cohort-r(min)+1
replace cohort = round(cohort,3)

gen lcons = log(cons)
gen lincome = log(income)
keep lcons lincome cohort age unempl ret famsize numearn year
replace age = round(age)
replace famsize = round(famsize)
replace numearn = round(numearn)

reg lcons i.famsize i.age i.year i.cohort ret
predict plcons if e(sample)

collapse (median) lcons (median) plcons, by (age)
label var age "age"
scatter (plcons age), msymbol(d s d s) mcolor(midblue forest_green edkblue olive) title("CEX consumption data 1980-2002") subtitle("log consumption in 1984 $ controlling for cohort, family size, and time effects", size(large)) xtitle("age", size(large)) msymbol(D S) ytitle("")  
*scatter plcons age || fpfit plcons age, msymbol(d s d s) mcolor(midblue forest_green edkblue olive) title("CEX consumption data 1980-2002") subtitle("log consumption in 1984 $ controlling for cohort, family size, and time effects", size(large)) xtitle("age", size(large)) msymbol(D S) ytitle("") legend(order(1 "consumption" 2 "predicted consumption") size(large))  
///saving("$direct/CEXcons.png", replace)
