% Simulating the inattention model
clear all; close all; beep off
h = 1/12;  T = 30*1/h; PC = 0;
% parameter values
beta = 0.98^h; eta = 1; lambda = 2.5; gamma = 0.5; mur = h*7/100; sigmar = sqrt(h)*20/100; rf = h*2/100; rd = h*0/100;
% calculating expectations
n = 60; [r_grid w_grid] = lgwt(n,mur-8*sigmar,mur+8*sigmar); r_grid = flipdim(r_grid,1); w_grid = flipdim(w_grid,1);
Fr_grid = normcdf(r_grid,mur,sigmar); wr_grid = normpdf(r_grid',mur,sigmar).*w_grid';

% Comparison to prospective news utility over a signal and the expectation of which conditional on a signal
sigmaeps = sigmar/2; ut = @(c) log(c); up = @(c) c.^(-1); mu = @(x) eta.*x.*(x>=0)+eta.*lambda.*x.*(x<0);
nu_conds = @(Rtilde) gamma*beta*quadv(@(y) quadv(@(x) normpdf(x,mur,sigmar).*normpdf(y,0,sigmaeps).*mu(ut(exp(log(Rtilde)-y))-ut(exp(x))),mur-8*sigmar,mur+8*sigmar),-8*sigmaeps,8*sigmaeps);
nu_signal = @(Rtilde) gamma*beta*quadv(@(y) quadv(@(xz) normpdf(xz,mur,sqrt(sigmar^2+sigmaeps^2)).*normpdf(y,0,sigmaeps).*mu(ut(exp(log(Rtilde)-y))-ut(exp(xz-y))),mur-8*sqrt(sigmar^2+sigmaeps^2),mur+8*sqrt(sigmar^2+sigmaeps^2)),-8*sigmaeps,8*sigmaeps);
r_vec = [mur-2*sigmar:16*sigmar/80:mur+8*sigmar]'; critical_value = max(abs(nu_conds(exp(r_vec))-nu_signal(exp(r_vec)))); count = 0;

% Inattention model policy functions:
% standard model:
beta_sum_vec = beta*ones(T,1); for i=2:T; beta_sum_vec(i) = beta_sum_vec(i-1)+beta.^i; end; beta_sum = @(i) beta_sum_vec(i.*(i>0)+1.*(i==0))-beta.*(i==0);
rhoTC = @(i) 1./(1+beta_sum(i)); alphaTC = max(min((mur-rf)./sigmar.^2,1),0);
% news-utility model
rho = @(i,Fr) 1./(1+beta_sum(i).*(1+gamma.*(eta.*Fr+eta.*lambda.*(1-Fr)))./(1+eta.*Fr+eta.*lambda.*(1-Fr)));
alpha = @(i,j1,Fr) max(min((mur-rf+(1+gamma.*beta_sum(i-j1))./(1+beta_sum(i-j1)).*sqrt(j1)./j1.*sigmar.*gl_const./(1+gamma.*(eta.*Fr+eta.*lambda.*(1-Fr))))./sigmar.^2,1),0);
if PC == 1; rho = @(i,Fr) 1./(1+beta_sum(i).*(1+gamma.*(eta.*(lambda-1).*(2.*Fr-1)))./(1+eta.*(lambda-1).*(2*Fr-1)));
    alpha = @(i,j1,Fr) max(min((mur-rf+(1+gamma.*beta_sum(i-j1))./(1+beta_sum(i-j1)).*sqrt(j1)./j1.*sigmar.*gl_const./(1+gamma.*(eta.*(lambda-1).*(2.*Fr-1))))./sigmar.^2,1),0); end;
rho_in = zeros(T,T,T-1); % matrix for inattentive consumption: rows time until attentive j1, columns time last time attentive ij, third dim time today i
for i = 1:T-1; ijj = T-i; jj1 = i;
    for ij = 1:ijj
        for j1 = 1:jj1
            if rho_in(ij,j1,i)>0; 
            else threshold = 1; rho_in_start = nonzeros([rhoTC((i-j1+1:i-1)'.*(j1>1)).*(j1>1); rhoTC(i); rhoTC((i+1:i+ij-1)'.*(ij>1)).*(ij>1)]).*ones(ij+j1-1,1); rho_in_upd = zeros(ij+j1-1,1); rho_in_sum = sum(rho_in_start);
            while threshold>0.001*rhoTC(i) || 0>rho_in_sum || rho_in_sum>1
                threshold = 0;
                for k=1:ij+j1-1
                    rho_in_sum = sum(rho_in_start)-rho_in_start(k);
                    rho_in_upd(k) = 1./(1+beta^j1*(1+beta_sum(i-j1))).*max(min((1-rho_in_sum),1),0);
                    threshold = threshold+abs(rho_in_start(k)-rho_in_upd(k));
                    rho_in_start(k) = rho_in_upd(k);
                end;
            end;
            for k=1:ij+j1-1; rho_in(k,ij+j1-k,i+ij-k) = rho_in_start(k); end; % rho_in(ij,j1,i) = rho_in_start(ij);
            end; 
        end;
    end; i
end;
% save('rho_in.mat','rho_in','T','h','-v7.3');
% load('rho_in.mat','rho_in','T','h');
% Solving for optimal inattention intervals:
fin_grid = [];
for g=1:3; inatt_grid = zeros(20,T); gamma_grid = (0.5:0.5/19:1)'; lambda_grid = [2 2.5 3]; lambda = lambda_grid(g); % lambda_grid = (1:2/19:3)';
for gg=1:20; gamma = gamma_grid(gg,1);
gl_const = eta*(lambda-1)*intap(@(s) normpdf(s).*intap(@(stilde) normpdf(stilde).*(s-stilde),s,8),-8,8);
rho = @(i,Fr) 1./(1+beta_sum(i).*(1+gamma.*(eta.*Fr+eta.*lambda.*(1-Fr)))./(1+eta.*Fr+eta.*lambda.*(1-Fr)));
alpha = @(i,j1,Fr) max(min((mur-rf+(1+gamma.*beta_sum(i-j1))./(1+beta_sum(i-j1)).*sqrt(j1)./j1.*sigmar.*gl_const./(1+gamma.*(eta.*Fr+eta.*lambda.*(1-Fr))))./sigmar.^2,1),0);
lookup_mat = 100*ones(T,T); V = zeros(T,T); alpha_mat = zeros(T,T); % rows periods inattentive, columns periods to go
% find optimal j1 for all periods inattentive ij (state variable) (ji = 1 aget was attentive last period attentive)
i=0; for ji = 1:T-i; alpha_av = wr_grid*alpha(i+ji,ji,Fr_grid); alpha_av1 = wr_grid*(alpha(i+ji,ji,Fr_grid).*(1-alpha(i+ji,ji,Fr_grid))); V(ji,i+1) = ji*rf+alpha_av*(ji*mur-ji*rf)+alpha_av1*ji*sigmar^2/2+log(1)+alpha_av*sqrt(ji)*sigmar*gl_const;
            if alpha_av == 0; nolook = ji*rf+log(1); else nolook = ji*rd+log(1); end; % if he would not look in the last period his portfolio share would be zero
            if V(ji,i+1)>=nolook && alpha_av>0; lookup_mat(ji,i+1) = 1; alpha_mat(ji,i+1) = alpha_av; elseif V(ji,i+1)<nolook && alpha_av>0; V(ji,i+1) = nolook; lookup_mat(ji,i+1) = 0; alpha_mat(ji,i+1) = 0; else V(ji,i+1) = nolook; lookup_mat(ji,i+1) = 0; alpha_mat(ji,i+1) = alpha_av; end;
     end;
for i = 1:T-1;
    for ji = 1:T-i;
            % finding the value of looking today 
            for j1 = 1:i; alphaj1 = wr_grid*alpha(i,j1,Fr_grid); if alphaj1>0; break; else lookup_mat(j1,i+1-j1) = 0; end; end;
            if alphaj1 == 0; beta_rho_sumi = 0; for j=1:j1-1; beta_rho_sumi = beta_rho_sumi+beta^j*(rf*(1+beta_sum(i-j))+log(rhoTC(i-j))+beta_sum(i-j)*log(1-rhoTC(i-j))); end; % sum up utils from T-i+1 to T-1
                beta_rho_sum = beta_rho_sumi+beta^j1*(rf*(1+beta_sum(i-j1))+log(rhoTC(i-j1))); % plus utils in T
            else beta_rho_sum1 = 0; beta_rho_sum = beta^j1*V(j1,i+1-j1); % utils after he looks
                for j=1:j1-1; beta_rho_sum = beta_rho_sum+beta^j*(j*rd+log(rho_in(j,j1-j,i-j))); beta_rho_sum1 = beta_rho_sum1+rho_in(j,j1-j,i-j); end; % utils until he looks
                beta_rho_sum = beta_rho_sum+beta^j1*(1+beta_sum(i-j1))*log(1-beta_rho_sum1);
            end; jj1 = j1+1; better = 0;
            if jj1<=i; while better == 0; j1 = jj1; alphaj1 = wr_grid*alpha(i,j1,Fr_grid);
                            if alphaj1 == 0; beta_rho_sumi = 0; for j=1:j1-1; beta_rho_sumi = beta_rho_sumi+beta^j*(rf*(1+beta_sum(i-j))+log(rhoTC(i-j))+beta_sum(i-j)*log(1-rhoTC(i-j))); end; % sum up utils from T-i+1 to T-1
                                beta_rho_sumb = beta_rho_sumi+beta^j1*(rf*(1+beta_sum(i-j1))+log(rhoTC(i-j1))); % plus utils in T
                                if beta_rho_sumb>beta_rho_sum; lookup_mat(jj1-1,i+1-(jj1-1)) = 0; beta_rho_sum = beta_rho_sumb; end;
                            else beta_rho_sum1 = 0; beta_rho_sumb = beta^j1*V(j1,i+1-j1); for j=1:j1-1; beta_rho_sumb = beta_rho_sumb+beta^j*(j*rd+log(rho_in(j,j1-j,i-j))); beta_rho_sum1 = beta_rho_sum1+rho_in(j,j1-j,i-j); end; 
                                beta_rho_sumb = beta_rho_sumb+beta^j1*(1+beta_sum(i-j1))*log(1-beta_rho_sum1);
                                if beta_rho_sumb>beta_rho_sum; lookup_mat(jj1-1,i+1-(jj1-1)) = 0; beta_rho_sum = beta_rho_sumb; end; 
                            end; jj1 = j1+1; if j1 == i; better = 1; end;
                        end;        
            end; 
            % final value of looking today
            alpha_av = wr_grid*alpha(i+ji,ji,Fr_grid); alpha_av1 = wr_grid*(alpha(i+ji,ji,Fr_grid).*(1-alpha(i+ji,ji,Fr_grid))); rho_av = wr_grid*rho(i,Fr_grid);
            if alpha_av == 0; V(ji,i+1) = ji*rf*(1+beta_sum(i))+log(rhoTC(i))+beta_sum(i)*log(1-rhoTC(i))+beta_rho_sum;
            else V(ji,i+1) = ji*(rf+alpha_av*(mur-rf)+alpha_av1*sigmar^2/2)*(1+beta_sum(i))+log(rho_av)+alpha_av*(1+gamma*beta_sum(i))*sqrt(ji)*sigmar*gl_const+beta_sum(i)*log(1-rho_av)+beta_rho_sum;
            end;
         % finding the value of not looking today 
            for j1 = 1:i; alphaji1 = wr_grid*alpha(i+ji,ji+j1,Fr_grid); if alphaji1>0; break; else lookup_mat(ji+j1,i+1-j1) = 0; end; end;
            if alphaji1 == 0; beta_rho_sumi = 0; for j=1:j1-1; beta_rho_sumi = beta_rho_sumi+beta^j*(rf*(1+beta_sum(i-j))+log(rhoTC(i-j))+beta_sum(i-j)*log(1-rhoTC(i-j))); end; % utils until period T-1
                beta_rho_sum = beta_rho_sumi+beta^j1*(rf*(1+beta_sum(i-j1))+log(rhoTC(i-j1))); % utils in period T
                % final value of not looking today
                nolook = ji*rf*(1+beta_sum(i))+log(rhoTC(i))+beta_sum(i)*log(1-rhoTC(i))+beta_rho_sum;
            else beta_rho_sum1 = 0; beta_rho_sum = beta^j1*V(j1+ji,i+1-j1); % value after he looked next time
                % second value until he looks
                for j=1:j1-1; beta_rho_sum = beta_rho_sum+beta^j*(j*rd+log(rho_in(j+ji,j1-j,i-j))); end; % utils until he looks
                for j = 1:ji+j1-1; beta_rho_sum1 = beta_rho_sum1+rho_in(j,ji+j1-j,i+ji-j); end; beta_rho_sum = beta_rho_sum+beta^j1*(1+beta_sum(i-j1))*log(1-beta_rho_sum1); % utils after he looks
                % final value of not looking today
                nolook = ji*rd+log(rho_in(ji,j1,i))+beta_rho_sum;
            end; jj1 = j1+1; better = 0;
            if jj1<=i; while better == 0; j1 = jj1; alphaji1 = wr_grid*alpha(i+ji,ji+j1,Fr_grid); % if lookup_mat(ji+j1,i+1-j1) == 1 && alphaji1==0; lookup_mat(ji+j1,i+1-j1) = 0; elseif lookup_mat(ji+j1,i+1-j1) == 1  && alphaji1>0; alphaji1 = wr_grid*alpha(i+ji,ji+j1,Fr_grid); break; elseif j1 == i && lookup_mat(ji+j1,i+1-j1) == 0; alphaji1 = 0;  end; end;
                           if alphaji1 == 0; beta_rho_sumi = 0; for j=1:j1-1; beta_rho_sumi = beta_rho_sumi+beta^j*(rf*(1+beta_sum(i-j))+log(rhoTC(i-j))+beta_sum(i-j)*log(1-rhoTC(i-j))); end; % utils until period T-1
                               beta_rho_sumb = beta_rho_sumi+beta^j1*(rf*(1+beta_sum(i-j1))+log(rhoTC(i-j1))); % utils in period T
                               beta_rho_sumb = ji*rf*(1+beta_sum(i))+log(rhoTC(i))+beta_sum(i)*log(1-rhoTC(i))+beta_rho_sumb;
                                if beta_rho_sumb>nolook; lookup_mat(ji+jj1-1,i+1-(jj1-1)) = 0; nolook = beta_rho_sumb; end;
                            else beta_rho_sum1 = 0; beta_rho_sumb = beta^j1*V(j1+ji,i+1-j1);
                                for j=1:j1-1; beta_rho_sumb = beta_rho_sumb+beta^j*(j*rd+log(rho_in(j+ji,j1-j,i-j))); end; % utils until he looks
                                for j = 1:ji+j1-1; beta_rho_sum1 = beta_rho_sum1+rho_in(j,ji+j1-j,i+ji-j); end; beta_rho_sumb = beta_rho_sumb+beta^j1*(1+beta_sum(i-j1))*log(1-beta_rho_sum1);
                                beta_rho_sumb = ji*rd+log(rho_in(ji,j1,i))+beta_rho_sumb;
                                if beta_rho_sumb>nolook; lookup_mat(ji+jj1-1,i+1-(jj1-1)) = 0; nolook = beta_rho_sumb; end; 
                            end; jj1 = j1+1; if j1 == i; better = 1; end; 
                        end; 
            end;
            % finally comparing utility from looking and not looking
            if V(ji,i+1)>nolook; lookup_mat(ji,i+1) = 1; V_signal_check = V(ji,i+1); % he decides to look
            else V_signal_check = V(ji,i+1); V(ji,i+1) = nolook; lookup_mat(ji,i+1) = 0; 
            end;
            % would his decision be affected by the presence of a signal
            if abs(V_signal_check-nolook)<critical_value; count = count+1; end;
    end; i
end;
disp('percent of decisions potentially affected by signal:'); disp(count/(i*i)*100);

% Figuring out average intervals of inattention starting from first period
av_inattention = zeros(1,T); av_alpha = av_inattention; av_alpha_cond = av_inattention; av_part = av_alpha; 
% when will he first invest?
t = 1; for j1 = 1:T-t; if lookup_mat(j1,T-(t+j1)+1) == 1; j1_first = j1; break; end; end; V_ffirst = zeros(j1_first,1);
V_first1 = beta*(rf*(1+beta_sum(T-1))+log(rhoTC(T-1))+beta_sum(T-1)*log(1-rhoTC(T-1))); V_first = V_first1; beta_rho_sum = 0; for j=2:j1-1; V_first = V_first+beta^j*(j*rd+log(rho_in(j,j1-j,T-j))); beta_rho_sum = beta_rho_sum+rho_in(j,j1-j,T-j); end; V_first = V_first+beta^j1*beta_sum(T-1-j1)*log(1-beta_rho_sum)+beta^j1*V(j1,T-(t+j1)+1); V_ffirst(1) = V_first;
for t = 2:j1_first; for j = 2:t; V_first1 = V_first1+beta^j*(j*rf*(1+beta_sum(T-j))+log(rhoTC(T-j))+beta_sum(T-j)*log(1-rhoTC(T-j))); end; 
    for j1 = 1:T-t; if lookup_mat(j1,T-(t+j1)+1) == 1; j1_first = j1; break; end; end;
    V_first = V_first1; beta_rho_sum = 0; for j=2:j1-1; V_first = V_first+beta^j*(j*rd+log(rho_in(j,j1-j,T-j))); beta_rho_sum = beta_rho_sum+rho_in(j,j1-j,T-j); end; V_first = V_first+beta^j1*beta_sum(T-1-j1)*log(1-beta_rho_sum)+beta^j1*V(j1,T-(t+j1)+1); V_ffirst(t) = V_first;
end;
[Vmax i_start] = max(V_ffirst); t = i_start; num = 1;
while t<T; for j1 = 1:T-t; if lookup_mat(j1,T-(t+j1)+1) == 1;
                               av_inattention(num) = j1; av_alpha(num) = wr_grid*alpha(T-t,j1,Fr_grid); av_alpha_cond(num) = wr_grid*alpha(T-t,j1,Fr_grid)./sum(wr_grid(alpha(T-t,j1,Fr_grid)>0)); if alpha(T-t,j1,1)>0; av_part(num) = 1; else av_part(num) = fsolve(@(p) alpha(T-t,j1,p),0); end; break; 
                           end; 
           end; num = num+1; t = t+j1;
end; av_inattention(num-1:num-1+j1-1) = ones(1,j1); av_alpha(num-1:num+j1-2) = 0; av_part(num-1:num+j1-2) = 0; num = num+j1-2;
av_inattention(1:num-1)
av_alpha_cond = av_alpha_cond(1:num-1)
av_part = av_part(1:num-1)
i_start
inatt_grid(gg,:) = av_inattention;
end
fin_grid = [fin_grid inatt_grid(:,1)]
end

plot(gamma_grid,fin_grid(:,1),'*-',gamma_grid,fin_grid(:,2),'+-',gamma_grid,fin_grid(:,3),'x-','LineWidth',2)
hh = legend('months inattentive $j_1$ for $\eta(\lambda-1)=1$','months inattentive $j_1$ for $\eta(\lambda-1)=1.5$','months inattentive $j_1$ for $\eta(\lambda-1)=2$','Location','NorthWest'); set(hh,'Interpreter','latex'); legend BOXOFF
LEG = findobj(hh,'type','text'); set(LEG,'FontSize',12)
xlabel('prospective news discounting \gamma','FontSize',28); ylabel('average interval of inattention','FontSize',28); 
saveas(gcf,'j1gammaplot.png')
