% Life-cycle portfolio-choice model semi-analytical solution
clear all; close all; beep off; pause off
% parameter values
h = 1; T = 54*1/h; beta = 0.98; theta = 1; eta = 1; lambda = 3; gamma = 0.8; rf = h*0.0086; sigmar = sqrt(h)*19.4/100; mur = h*(rf*100+6.33)/100; Rf = exp(rf); par=[Rf mur sigmar beta theta eta lambda gamma];
u = @(c) 1/(1-theta)*c.^(1-theta); up = @(c) c.^(-theta); if theta == 1; u = @(c) log(c); up = @(c) 1./c; end;
% calculating expectations
n = 20; [r_grid w_grid] = lgwt(n,mur-3*sigmar,mur+3*sigmar); r_grid = flipdim(r_grid,1); w_grid = flipdim(w_grid,1);
FR_grid = normcdf(r_grid,mur,sigmar); R_grid = exp(r_grid); w_grid = normpdf(r_grid',mur,sigmar).*w_grid'; options = optimset('Display','off');
% policy functions:
% standard model
Rp=@(R,alpha) Rf+alpha.*(R-Rf);
alphaTC = fsolve(@(alpha) w_grid*[((Rf+alpha*(R_grid-Rf)).^(-theta).*(R_grid-Rf))],(mur-log(Rf)+0.5*sigmar^2)/(sigmar^2*theta),options);
% news_utility model
alpha = @(R,FR) fsolve(@(alpha) gamma*beta*w_grid*[Rp(R_grid,alpha).^(-theta).*(R_grid-Rf)]*(eta*FR+eta*lambda*(1-FR))...
      +beta*w_grid*[Rp(R_grid,alpha).^(-theta).*(R_grid-Rf)]+beta*eta*(lambda-1)*w_grid*[w_grid*([[kron((Rp(R_grid,alpha).^(-theta).*(R_grid-Rf))',ones(n,1))-kron(Rp(R_grid,alpha).^(-theta).*(R_grid-Rf),ones(1,n))].*(kron(R_grid',ones(n,1))<kron(R_grid,ones(1,n)))])]',alphaTC,options);
alpha_grid = zeros(length(R_grid),T); rho_grid = zeros(length(R_grid),T); rho_grid(:,1) = ones(length(R_grid),1);
for i=1:n
    alpha_grid(i,2) = alpha(R_grid(i),FR_grid(i));
end; alpha_grid(:,2) = min(max(alpha_grid(:,2),0),1);
Rp_grid = Rf+kron(alpha_grid(:,2)',ones(n,1)).*(kron(R_grid,ones(1,n))-Rf); psi_grid = [zeros(T-1,length(R_grid));[beta*w_grid*Rp_grid.^(1-theta)];zeros(1,length(R_grid))];
for j = 1:n; psi_grid(T,j) = psi_grid(T,j)+beta*eta*(lambda-1)*w_grid*[w_grid*[(kron(Rp_grid(:,j).^(1-theta)',ones(n,1))-kron(Rp_grid(:,j).^(1-theta),ones(1,n))).*(kron(Rp_grid(:,j)',ones(n,1))<kron(Rp_grid(:,j),ones(1,n)))]]'; 
end; Q_grid = [zeros(T-1,length(R_grid));[beta*w_grid*Rp_grid.^(1-theta)];zeros(1,length(R_grid))]; QTC_grid = [zeros(T-1,1);[beta*w_grid*Rp(R_grid,alphaTC).^(1-theta)];0];
for i=1:T-1;
    % standard model
    rhoTC = 1./(1+QTC_grid(T-i+1)^(1/theta)); rhoTC_grid(:,i+1) = rhoTC*ones(n,1);
    QTC_grid(T-i) = beta*w_grid*[Rp(R_grid,alphaTC).^(1-theta).*rhoTC^(1-theta)]+beta*w_grid*[Rp(R_grid,alphaTC).^(1-theta).*(1-rhoTC)^(1-theta)].*QTC_grid(T-i+1);
    % news-utility model
    rho_grid(:,i+1) = 1./(1+(psi_grid(T-i+1,:)'+gamma*Q_grid(T-i+1,:)'.*(eta*FR_grid+eta*lambda*(1-FR_grid))).^(1/theta).*(1+eta*FR_grid+eta*lambda*(1-FR_grid)).^(-1/theta));
    alpha = @(R,FR,alpha_start) fsolve(@(alpha) gamma*w_grid*[Rp(R_grid,alpha).^(-theta).*(R_grid-Rf).*rho_grid(:,i+1).^(1-theta)+Rp(R_grid,alpha).^(-theta).*(R_grid-Rf).*(1-rho_grid(:,i+1)).^(1-theta).*Q_grid(T-i+2,:)']*(eta*FR+eta*lambda*(1-FR))...
           +beta*w_grid*[Rp(R_grid,alpha).^(-theta).*(1-rho_grid(:,i+1)).^(1-theta).*(R_grid-Rf)]...
           +beta*eta*(lambda-1)*w_grid*[w_grid*([[kron((Rp(R_grid,alpha).^(-theta).*(1-rho_grid(:,i+1)).^(1-theta).*(R_grid-Rf))',ones(n,1))-kron(Rp(R_grid,alpha).^(-theta).*rho_grid(:,i+1).^(1-theta).*(R_grid-Rf),ones(1,n))].*(kron(R_grid',ones(n,1))<kron(R_grid,ones(1,n)))])]'...
           +beta*gamma*eta*(lambda-1)*w_grid*([w_grid*([[kron((Rp(R_grid,alpha).^(-theta).*Q_grid(T-i+2,:)'.*(1-rho_grid(:,i+1)).^(1-theta).*(R_grid-Rf))',ones(n,1))-kron(Rp(R_grid,alpha).^(-theta).*Q_grid(T-i+2,:)'.*(1-rho_grid(:,i+1)).^(1-theta).*(R_grid-Rf),ones(1,n))].*(kron(R_grid',ones(n,1))<kron(R_grid,ones(1,n)))])]')...
           +beta*w_grid*[(1-rho_grid(:,i+1)).^(1-theta).*Rp(R_grid,alpha).^(-theta).*(R_grid-Rf).*psi_grid(T-i+2,:)'],alpha_start,options);
    for j=1:length(R_grid)
        alpha_grid(j,i+2) = alpha(R_grid(j),FR_grid(j),alpha_grid(j,i+1));
    end; alpha_grid(:,i+2) = min(max(alpha_grid(:,i+2),0),1); Rp_grid = Rf+kron(alpha_grid(:,i+2)',ones(n,1)).*(kron(R_grid,ones(1,n))-Rf);
    Rp_grid = Rf+kron(alpha_grid(:,i+2)',ones(n,1)).*(kron(R_grid,ones(1,n))-Rf);
    psi_grid(T-i,:) = beta*w_grid*[Rp_grid.^(1-theta)].*rho_grid(:,i+1)'.^(1-theta)+beta*w_grid.*(1-rho_grid(:,i+1)').^(1-theta)*[Rp_grid.^(1-theta)].*psi_grid(T-i+1,:);
    for j = 1:n; psi_grid(T-i,j) = psi_grid(T-i,j)+beta*eta*(lambda-1)*w_grid*[w_grid*[(kron(Rp_grid(:,j).^(1-theta)'.*rho_grid(:,i+1).^(1-theta)',ones(n,1))-kron(Rp_grid(:,j).^(1-theta).*rho_grid(:,i+1).^(1-theta),ones(1,n))).*(kron(Rp_grid(:,j)',ones(n,1))<kron(Rp_grid(:,j),ones(1,n)))]]'...
            +beta*gamma*eta*(lambda-1)*w_grid*([w_grid*[(kron(Rp_grid(:,j).^(1-theta)'.*Q_grid(T-i+1,:).*(1-rho_grid(:,i+1)).^(1-theta)',ones(n,1))-kron(Rp_grid(:,j).^(1-theta).*Q_grid(T-i+1,:)'.*(1-rho_grid(:,i+1)).^(1-theta),ones(1,n))).*(kron(Rp_grid(:,j)',ones(n,1))<kron(Rp_grid(:,j),ones(1,n)))]]'); 
    end; Q_grid(T-i,:) = beta*w_grid*[Rp_grid.^(1-theta)].*rho_grid(:,i+1)'.^(1-theta)+beta*w_grid*[Rp_grid.^(1-theta)].*(1-rho_grid(:,i+1)').^(1-theta).*Q_grid(T-i+1,:);
end
rho_grid(:,T+1) = 1./(1+(psi_grid(1,:)'+gamma*Q_grid(1,:)'.*(eta*FR_grid+eta*lambda*(1-FR_grid))).^(1/theta).*(1+eta*FR_grid+eta*lambda*(1-FR_grid)).^(-1/theta));
rhoTC = 1./(1+QTC_grid(1)^(1/theta)); rhoTC_grid(:,T+1) = rhoTC*ones(n,1);

% attitudes towards wealth bets
W_bar = 300000; Psi = @(A) u(A).*w_grid*psi_grid(1,:)'; Phi = @(A) u(A).*w_grid*Q_grid(1,:)'; PsiTC = @(A) u(A).*QTC_grid(1);
% standard
a = @(G,L) 0.5*PsiTC(W_bar+G)+0.5*PsiTC(W_bar-L);
r = Psi(W_bar); G_fun = @(L) fsolve(@(G) (a(G,L)-r)*10e8,L,optimset('MaxFunEvals',1000)); G_vec_s = [G_fun(10) G_fun(200) G_fun(1000) G_fun(5000) G_fun(50000) G_fun(100000)]
% contemporaneous
a = @(G,L) 0.5*eta*(u(W_bar+G)-u(W_bar))+0.5*eta*lambda*(u(W_bar-L)-u(W_bar))+0.5*u(W_bar+G)+0.5*u(W_bar-L);
r = u(W_bar); G_fun = @(L) fsolve(@(G) (a(G,L)-r)*10e10,L); G_vec_c = [G_fun(10) G_fun(200) G_fun(1000) G_fun(5000) G_fun(50000) G_fun(100000)]
% prospective
a = @(G,L) gamma*0.5*eta*(Phi(W_bar+G)-Phi(W_bar))+gamma*0.5*eta*lambda*(Phi(W_bar-L)-Phi(W_bar))+0.5*Psi(W_bar+G)+0.5*Psi(W_bar-L);
r = Psi(W_bar); G_fun = @(L) fsolve(@(G) (a(G,L)-r)*10e8,L,optimset('MaxFunEvals',1000)); G_vec_p = [G_fun(10) G_fun(200) G_fun(1000) G_fun(5000) G_fun(50000) G_fun(100000)]

alphaTC = alphaTC*ones(T,1);
% life-cycle consumption and risky asset holdings
psi = psi_grid; Q = Q_grid; QTC = QTC_grid;
N=10000; W0 = 100; x1 = zeros(N,1); x2 = zeros(N,1); C = zeros(T,N);  WC = zeros(T,N); alpha = zeros(T,N); alphanu = zeros(T,1); RA = zeros(T,N); RAnu = zeros(T,1); RAb = zeros(T,N); RAbnu = zeros(T,1); Cnu = zeros(T,1);  WCnu = zeros(T,1);  CTC = zeros(T,N); WCTC = zeros(T,N);  RATC = zeros(T,N); RATCnu = zeros(T,1); RAbTC = zeros(T,N); RAbTCnu = zeros(T,1); CTCnu = zeros(T,1);  WCTCnu = zeros(T,1); 
for k=1:N
    r = mur+sigmar*randn(T,1);
    j=1; W = [W0*exp(r(1)); zeros(T-1,1)]; Wnu = [W0*exp(mur); zeros(T-1,1)]; WTC = W; WTCnu = Wnu; 
    psi_fun = @(r) spline(r_grid,psi(j+1,:)',r); Q_fun = @(r) spline(r_grid,Q(j+1,:)',r); alpha_fun = @(r) spline(r_grid,alpha_grid(:,T-j+1),r);
    rho = @(r) 1./(1+(psi_fun(r)+gamma*Q_fun(r).*(eta*normcdf(r,mur,sigmar)+eta*lambda*(1-normcdf(r,mur,sigmar)))).^(1/theta).*(1+eta*normcdf(r,mur,sigmar)+eta*lambda*(1-normcdf(r,mur,sigmar))).^(-1/theta));
    C(j,k) = rho(r(j))*W(j); WC(j,k) = W(j)-C(j,k); alpha(j,k) = alpha_fun(r(j)); RA(j,k) = W(j)*(1-rho(r(j)))*alpha_fun(r(j)); RAb(j,k) = W(j)*(1-rho(r(j)))*alpha_fun(r(j))*exp(r(j+1));
    Cnu(j) = rho(mur)*Wnu(j); WCnu(j) = Wnu(j)-Cnu(j); alphanu(j) = alpha_fun(mur); RAnu(j) = Wnu(j)*(1-rho(mur))*alpha_fun(mur); RAbnu(j) = Wnu(j)*(1-rho(mur))*alpha_fun(mur)*exp(mur);
    psi_fun = @(r) spline(r_grid,psi(j+1,:)',r); Q_fun = @(r) spline(r_grid,Q(j+1,:)',r);
    rho = @(r) 1./(1+(psi_fun(r)+gamma*Q_fun(r).*eta*(lambda-1)*(1-2*normcdf(r,mur,sigmar)))^(1/theta).*(1+eta*(lambda-1)*(1-2*normcdf(r,mur,sigmar)))^(-1/theta));
    rhoTC = 1./(1+QTC(j+1)^(1/theta)); RATC(j,k) = WTC(j)*(1-rhoTC)*alphaTC(j); RATCnu(j) = WTCnu(j)*(1-rhoTC)*alphaTC(j); RAbTC(j,k) = WTC(j)*(1-rhoTC)*alphaTC(j)*exp(r(j+1));
    CTC(j,k) = rhoTC*WTC(j); WCTC(j,k) = WTC(j)-CTC(j,k); RAbTCnu(j) = WTCnu(j)*(1-rhoTC)*alphaTC(j)*exp(mur);
    CTCnu(j) = rhoTC*WTCnu(j); WCTCnu(j) = WTCnu(j)-CTCnu(j); x1(k) = r(3); x2(k) = r(2);
    for j=2:T-1
        W(j) = WC(j-1,k)*Rp(exp(r(j)),alpha(j-1)); Wnu(j) = WCnu(j-1)*Rp(exp(mur),alphanu(j-1)); WTC(j) = WCTC(j-1,k)*Rp(exp(r(j)),alphaTC(j-1)); WTCnu(j) = WCTCnu(j-1)*Rp(exp(mur),alphaTC(j-1));
        psi_fun = @(r) spline(r_grid,psi(j+1,:)',r); Q_fun = @(r) spline(r_grid,Q(j+1,:)',r); alpha_fun = @(r) spline(r_grid,alpha_grid(:,T-j+1),r);
        rho = @(r) 1./(1+(psi_fun(r)+gamma*Q_fun(r).*(eta*normcdf(r,mur,sigmar)+eta*lambda*(1-normcdf(r,mur,sigmar)))).^(1/theta).*(1+eta*normcdf(r,mur,sigmar)+eta*lambda*(1-normcdf(r,mur,sigmar))).^(-1/theta));
        C(j,k) = rho(r(j))*W(j); WC(j,k) = W(j)-C(j,k); alpha(j,k) = alpha_fun(r(j)); RA(j,k) = W(j)*(1-rho(r(j)))*alpha_fun(r(j)); RAb(j,k) = W(j)*(1-rho(r(j)))*alpha_fun(r(j))*exp(r(j+1));
        Cnu(j) = rho(mur)*Wnu(j); WCnu(j) = Wnu(j)-Cnu(j); alphanu(j) = alpha_fun(mur); RAnu(j) = Wnu(j)*(1-rho(mur))*alpha_fun(mur); RAbnu(j) = Wnu(j)*(1-rho(mur))*alpha_fun(mur)*exp(mur);
        rhoTC = 1./(1+QTC(j+1)^(1/theta)); RATC(j,k) = WTC(j)*(1-rhoTC)*alphaTC(j); RATCnu(j) = WTCnu(j)*(1-rhoTC)*alphaTC(j); 
        CTC(j,k) = rhoTC*WTC(j); WCTC(j,k) = WTC(j)-CTC(j,k);  RAbTC(j,k) = WTC(j)*(1-rhoTC)*alphaTC(j)*exp(r(j+1));
        CTCnu(j) = rhoTC*WTCnu(j); WCTCnu(j) = WTCnu(j)-CTCnu(j); RAbTCnu(j) = WTCnu(j)*(1-rhoTC)*alphaTC(j)*exp(mur);
    end
    j=T; W(j) = WC(j-1,k)*Rp(exp(r(j)),alpha(j-1)); Wnu(j) = WCnu(j-1)*Rp(exp(mur),alphanu(j-1)); WTC(j) = WCTC(j-1,k)*Rp(exp(r(j)),alphaTC(j-1)); WTCnu(j) = WCTCnu(j-1)*Rp(exp(mur),alphaTC(j-1));
    rho = @(r) 1; C(j,k) = rho(r(j))*W(j); WC(j,k) = W(j)-C(j,k); alpha(j,k) = 0;
    Cnu(j) = rho(mur)*Wnu(j); WCnu(j) = Wnu(j)-Cnu(j); alphanu(j) = 0;
    rhoTC = 1; CTC(j,k) = rhoTC*WTC(j); WCTC(j,k) = WTC(j)-CTC(j,k);
    CTCnu(j) = rhoTC*WTCnu(j); WCTCnu(j) = WTCnu(j)-CTCnu(j);
    x1(k) = r(3); x2(k) = r(2);
end
plot(1:T,10*1/log(Cnu(1)).*log(Cnu),'*-',1:T,10*1/log(Cnu(1)).*log(mean(C,2)),'+-',1:T,10*1/log(Cnu(1)).*log(CTCnu),'x-',1:T,10*1/log(Cnu(1)).*log(mean(CTC,2)),'.-','LineWidth',2); axis([0 60 8 16]);
h = legend('news utility profile for $r_t=\mu$ $\forall t$','news utility sample average','standard profile for $r_t=\mu$ $\forall t$','standard sample average','Location','South'); set(h,'Interpreter','latex'); legend BOXOFF
LEG = findobj(h,'type','text'); set(LEG,'FontSize',12)
ylabel('consumption','FontSize',28); xlabel('time t=1,...,T','FontSize',28);
saveas(gcf,'Ccheck.png')
